@@ -48,6 +48,7 @@
 #define SYS_NAME_ID_hp_ux11i             416
 #define SYS_NAME_ID_ia64_hpux1122	 417
 #define SYS_NAME_ID_ia64_hpux1123	 418
+#define SYS_NAME_ID_hp_ux1123		 419
 
 #define SYS_NAME_ID_mac2_51		 500
 #define SYS_NAME_ID_mac_aux10		 501
@@ -67,6 +68,7 @@
 #define SYS_NAME_ID_rs_aix51		 704
 #define SYS_NAME_ID_rs_aix43		 705
 #define SYS_NAME_ID_rs_aix52		 706
+#define SYS_NAME_ID_rs_aix53		 707
 
 #define SYS_NAME_ID_sun3_411		 906
 #define SYS_NAME_ID_sun3x_411		 912
@@ -128,7 +130,8 @@
 #define SYS_NAME_ID_ncrx86_30		1301
 
 #define SYS_NAME_ID_i386_nt35		1400
-#define SYS_NAME_ID_i386_win95	1401
+#define SYS_NAME_ID_i386_win95	        1401
+#define SYS_NAME_ID_i386_w2k            1402
 
 #define SYS_NAME_ID_i386_linux2		1500
 #define SYS_NAME_ID_i386_linux22	1501
@@ -154,10 +157,12 @@
 #define SYS_NAME_ID_s390x_linux2         1903
 #define SYS_NAME_ID_s390x_linux22        1904
 #define SYS_NAME_ID_s390x_linux24        1905
+#define SYS_NAME_ID_s390x_linux26        1906
 
 #define SYS_NAME_ID_alpha_linux_2       2000
 #define SYS_NAME_ID_alpha_linux_22      2001
 #define SYS_NAME_ID_alpha_linux_24      2002
+#define SYS_NAME_ID_alpha_linux_26      2003
 
 #define SYS_NAME_ID_i386_fbsd_42        2100
 #define SYS_NAME_ID_i386_fbsd_43        2101
@@ -169,6 +174,7 @@
 #define SYS_NAME_ID_i386_fbsd_51        2106
 #define SYS_NAME_ID_i386_fbsd_52        2107
 #define SYS_NAME_ID_i386_fbsd_53        2108
+#define SYS_NAME_ID_i386_fbsd_60        2112
 
 #define SYS_NAME_ID_ia64_linux2		2200
 #define SYS_NAME_ID_ia64_linux22	2201
@@ -198,6 +204,7 @@
 #define SYS_NAME_ID_sparc64_nbsd20	2523
 #define SYS_NAME_ID_macppc_nbsd20	2524
 #define SYS_NAME_ID_i386_nbsd21		2525
+#define SYS_NAME_ID_i386_nbsd30		2526
 
 #define SYS_NAME_ID_i386_obsd31		2600
 #define SYS_NAME_ID_i386_obsd32		2601
@@ -221,6 +228,12 @@
 #define SYS_NAME_ID_ppc64_linux24	2902
 #define SYS_NAME_ID_ppc64_linux26	2903
 
+#define SYS_NAME_ID_amd64_fbsd_53        3008
+
+#define SYS_NAME_ID_amd64_w2k           3100
+
+#define SYS_NAME_ID_i64_w2k             3200
+
 /*
  * Placeholder to keep system-wide standard flags since this file is included by all 
  * files (i.e in afs/param.h)
